/**************************************************************************\
*          Copyright (c) 1995 INRIA Sophia Antipolis, FRANCE.              *
*                                                                          *
* Permission to use, copy, modify, and distribute this material for any    *
* purpose and without fee is hereby granted, provided that the above       *
* copyright notice and this permission notice appear in all copies.        *
* WE MAKE NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS     *
* MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS   *
* OR IMPLIED WARRANTIES.                                                   *
\**************************************************************************/
/**************************************************************************\
*                                                                          *
*  File              : huffman.h                                           *
*  Last modification : 1995/2/15                                           *
*  Author            : Thierry Turletti                                    *
*--------------------------------------------------------------------------*
*  Description :  Huffman state arrays used by H.261 coder and decoder.    *
*                                                                          *
*                   - state_mba     -> Macroblock address.                 *
*                   - state_typem   -> Type information.                   *
*                   - state_cbc     -> Coded block pattern.                *
*                   - state_dvm     -> Motion vector.                      *
*                   - state_tcoeff  -> DCT coefficient.                    *
*                                                                          *
*--------------------------------------------------------------------------*
*        Name             |    Date   |          Modification              *
*--------------------------------------------------------------------------*
*                         |           | A new result is associated to the  *
*    Thierry Turletti     |  93/3/19  | state_mba table if there are more  *
*                         |           | than 15 zeroes.                    *
*--------------------------------------------------------------------------*
\**************************************************************************/
#undef EXTERN
#ifdef DATA_MODULE
#define EXTERN
#else
#define EXTERN extern
#endif


typedef struct{
	short int len;
	short unsigned int val;
	} TYPE_HUFF_TAB;


#define OFFSET_tab_mba  0

EXTERN double dummy;

EXTERN TYPE_HUFF_TAB idx_tab_mba[] 
#ifdef DATA_MODULE
 = {

{11, 0x000f},{ 1, 0x0001},{ 3, 0x0003},{ 3, 0x0002},
{ 4, 0x0003},{ 4, 0x0002},{ 5, 0x0003},{ 5, 0x0002},
{ 7, 0x0007},{ 7, 0x0006},{ 8, 0x000b},{ 8, 0x000a},
{ 8, 0x0009},{ 8, 0x0008},{ 8, 0x0007},{ 8, 0x0006},
{10, 0x0017},{10, 0x0016},{10, 0x0015},{10, 0x0014},
{10, 0x0013},{10, 0x0012},{11, 0x0023},{11, 0x0022},
{11, 0x0021},{11, 0x0020},{11, 0x001f},{11, 0x001e},
{11, 0x001d},{11, 0x001c},{11, 0x001b},{11, 0x001a},
{11, 0x0019},{11, 0x0018}
}
#endif
;
#define MAX_tab_mba  34


/*--------------------------------------------------------------------*/

#define OFFSET_tab_cbc  1

EXTERN TYPE_HUFF_TAB idx_tab_cbc[] 
#ifdef DATA_MODULE
 = {

{ 5, 0x000b},{ 5, 0x0009},{ 6, 0x000d},{ 4, 0x000d},
{ 7, 0x0017},{ 7, 0x0013},{ 8, 0x001f},{ 4, 0x000c},
{ 7, 0x0016},{ 7, 0x0012},{ 8, 0x001e},{ 5, 0x0013},
{ 8, 0x001b},{ 8, 0x0017},{ 8, 0x0013},{ 4, 0x000b},
{ 7, 0x0015},{ 7, 0x0011},{ 8, 0x001d},{ 5, 0x0011},
{ 8, 0x0019},{ 8, 0x0015},{ 8, 0x0011},{ 6, 0x000f},
{ 8, 0x000f},{ 8, 0x000d},{ 9, 0x0003},{ 5, 0x000f},
{ 8, 0x000b},{ 8, 0x0007},{ 9, 0x0007},{ 4, 0x000a},
{ 7, 0x0014},{ 7, 0x0010},{ 8, 0x001c},{ 6, 0x000e},
{ 8, 0x000e},{ 8, 0x000c},{ 9, 0x0002},{ 5, 0x0010},
{ 8, 0x0018},{ 8, 0x0014},{ 8, 0x0010},{ 5, 0x000e},
{ 8, 0x000a},{ 8, 0x0006},{ 9, 0x0006},{ 5, 0x0012},
{ 8, 0x001a},{ 8, 0x0016},{ 8, 0x0012},{ 5, 0x000d},
{ 8, 0x0009},{ 8, 0x0005},{ 9, 0x0005},{ 5, 0x000c},
{ 8, 0x0008},{ 8, 0x0004},{ 9, 0x0004},{ 3, 0x0007},
{ 5, 0x000a},{ 5, 0x0008},{ 6, 0x000c}
}
#endif
;
#define MAX_tab_cbc  64


/*--------------------------------------------------------------------*/

#define OFFSET_tab_dvm  -16

EXTERN TYPE_HUFF_TAB idx_tab_dvm[] 
#ifdef DATA_MODULE
 = {

{11, 0x0019},{11, 0x001b},{11, 0x001d},{11, 0x001f},
{11, 0x0021},{11, 0x0023},{10, 0x0013},{10, 0x0015},
{10, 0x0017},{ 8, 0x0007},{ 8, 0x0009},{ 8, 0x000b},
{ 7, 0x0007},{ 5, 0x0003},{ 4, 0x0003},{ 3, 0x0003},
{ 1, 0x0001},{ 3, 0x0002},{ 4, 0x0002},{ 5, 0x0002},
{ 7, 0x0006},{ 8, 0x000a},{ 8, 0x0008},{ 8, 0x0006},
{10, 0x0016},{10, 0x0014},{10, 0x0012},{11, 0x0022},
{11, 0x0020},{11, 0x001e},{11, 0x001c},{11, 0x001a}
}
#endif
;
#define MAX_tab_dvm  16


/*--------------------------------------------------------------------*/

#define OFFSET_tab_tcoeff  -1

EXTERN TYPE_HUFF_TAB idx_tab_tcoeff[] 
#ifdef DATA_MODULE
 = {

{ 2, 0x0002},{ 6, 0x0001},{ 2, 0x0003},{ 4, 0x0004},
{ 5, 0x0005},{ 7, 0x0006},{ 8, 0x0026},{ 8, 0x0021},
{10, 0x000a},{12, 0x001d},{12, 0x0018},{12, 0x0013},
{12, 0x0010},{13, 0x001a},{13, 0x0019},{13, 0x0018},
{13, 0x0017},{ 0, 0x0000},{ 3, 0x0003},{ 6, 0x0006},
{ 8, 0x0025},{10, 0x000c},{12, 0x001b},{13, 0x0016},
{13, 0x0015},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 4, 0x0005},{ 7, 0x0004},
{10, 0x000b},{12, 0x0014},{13, 0x0014},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 5, 0x0007},{ 8, 0x0024},
{12, 0x001c},{13, 0x0013},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 5, 0x0006},{10, 0x000f},
{12, 0x0012},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 6, 0x0007},{10, 0x0009},
{13, 0x0012},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 6, 0x0005},{12, 0x001e},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 6, 0x0004},{12, 0x0015},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 7, 0x0007},{12, 0x0011},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 7, 0x0005},{13, 0x0011},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 8, 0x0027},{13, 0x0010},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 8, 0x0023},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 8, 0x0022},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 8, 0x0020},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{10, 0x000e},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{10, 0x000d},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{10, 0x0008},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{12, 0x001f},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{12, 0x001a},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{12, 0x0019},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{12, 0x0017},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{12, 0x0016},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{13, 0x001f},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{13, 0x001e},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{13, 0x001d},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{13, 0x001c},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},{ 0, 0x0000},
{ 0, 0x0000},{ 0, 0x0000},{13, 0x001b}
}
#endif
;
#define MAX_tab_tcoeff  418


/*--------------------------------------------------------------------*/
